import argparse
from docx.shared import Cm
from titlecase import titlecase
from docxtpl import DocxTemplate, InlineImage


class ArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        raise Exception(message)
    

parser = ArgumentParser()
parser.add_argument("--template", help="Template number of the project file", required=True)
parser.add_argument("--session", help="Session year for which project file is being created", required=True)
parser.add_argument("--subject", help="Subject for which project file is being created", required=True)
parser.add_argument("--topic", help="Topic/name of the project", required=True)
parser.add_argument("--type", help="Type of the project", required=True)
parser.add_argument("--name", help="Name of person for whom project file is being created", required=True)
parser.add_argument("--gender", help="Gender of the person", required=True)
parser.add_argument("--std", help="Class in which the person studies", required=True)
parser.add_argument("--school", help="School name in which the person studies", required=True)
parser.add_argument("--rollNo", help="Roll number of the person", default="")
parser.add_argument("--teacher", help="Name of subject teacher", required=True)
parser.add_argument("--teacherDesig", help="Designation of the subject teacher", default="")
parser.add_argument("--principal", help="Name of principal of school", default=None)
parser.add_argument("--assistant", help="Name of the lab assistant", default=None)
parser.add_argument("--img", help="Image related to the topic of the project", default=None)
parser.add_argument("--logo", help="Logo of the school", default=None)
parser.add_argument("--index", help="Index of the project file", default=None)
parser.add_argument("--dir", help="Folder name in which files will be saved", required=True)

try:
    args = parser.parse_args()
    
    context = {
        'session': args.session,
        'subject': titlecase(args.subject.strip()),
        'topic': args.topic.strip(),
        'type': f"{args.subject.strip().upper()} {args.type.strip().upper()}",
        'name': titlecase(args.name.strip()),
        'pronoun': "his" if args.gender.lower() == "male" else "her",
        'class': args.std.strip().upper(),
        'school': titlecase(args.school.strip()),
        'roll_no': args.rollNo,
        'subject_teacher': titlecase(args.teacher.strip()),
        'teacher_desig': args.teacherDesig,
    }

    doc = DocxTemplate(f"projFilePageSet/templates/{args.template}.docx")

    if args.principal is not None:
        context['principal'] = f"I would like to extend my appreciation to {args.principal}, the Principal of our school, for always encouraging and supporting me. Her dedication to providing a good education and creating a positive learning environment is truly inspiring."
    if args.assistant is not None:
        context['lab_assistant'] = f"I also want to express my gratitude to {args.assistant.title()}, the lab assistant, for assisting me during the experiments."
    if args.index is not None:
        table_data = []
        for i, row in enumerate(args.index.split("\n")):
            content, pgno = row.split(",")
            table_data.append({"sno": i + 1, "content": titlecase(content.strip()), "pgno": pgno.strip()})
        context["table_data"] = table_data
    
    if args.img is not None:
        doc.replace_pic("image", args.img)
    if args.logo is not None:
        doc.replace_pic("logo", args.logo)

    doc.render(context)
    doc.save(f"{args.dir}/output.docx")

    print(True)
except Exception as e:
    print(e)
